#include <stdio.h>
#include <gmp.h>
#include "factor.h"
#include "RSAbase.h"
#include "md5.h"
#include "base64.h"
#include "apollo.h"


void RSAsign(char *signature,const char *mes,int meslen,const char *publickey,const char *secretkey,int keylen)
{
	RSAKeybase keypair;
	mpz_t m;
	
	if(meslen*4 > keylen*3)
		meslen = keylen*3/4;
	
	mpz_init(keypair->n);
	mpz_init(keypair->d);
	base64str_to_mpz(keypair->n,publickey,keylen);
	base64str_to_mpz(keypair->d,secretkey,keylen);
	
	mpz_init(m);
	mpz_import(m,meslen,-1,1,0,0,mes);
	
	RSAbase_encrypt(m,keypair);
	mpz_to_base64str(signature,keylen,m);
}

int RSAverify(const char *mes,int meslen,const char *testsignature,const char *publickey,int keylen)
{
	mpz_t m,n,c;
	
	if(meslen*4 > keylen*3)
		return -1;
	
	mpz_init(m);
	mpz_import(m,meslen,-1,1,0,0,mes);
	mpz_init(n);
	base64str_to_mpz(n,publickey,keylen);
	mpz_init(c);
	base64str_to_mpz(c,testsignature,keylen);
	
	RSAbase_decrypt(c,n);
	if(mpz_cmp(m,c) == 0)
		return 0;
	else
		return 1;
}

//
//	512bitp(apollo512-2)
static void make512pq(mpz_ptr p,mpz_ptr q,const char *keystr,int len)
{
	char *seedbuf;
	char hashs[64];
	int i;
	unsigned char *hashp;
	
	seedbuf = (char *)malloc(len+6);

	strncpy(seedbuf,keystr,len);
	seedbuf[len] = '\0';
	md5_buffer(seedbuf,len,&hashs[0]);
	seedbuf[len] = '\0';
	strcat(seedbuf,"pad1");
	md5_buffer(seedbuf,len+4,&hashs[16]);
	seedbuf[len] = '\0';
	strcat(seedbuf,"pad2");
	md5_buffer(seedbuf,len+4,&hashs[32]);
	seedbuf[len] = '\0';
	strcat(seedbuf,"pad3");
	md5_buffer(seedbuf,len+4,&hashs[48]);
	mpz_import(p,28,-1,1,0,0,&hashs[0]);
	mpz_import(q,36,-1,1,0,0,&hashs[28]);
	mpz_setbit(p,215);
	mpz_setbit(q,279);
	
	free(seedbuf);
}

int keycreate512(char *publickey,char *secretkey,const char *keystr,int len)
{
	mpz_t p,q;
	RSAKeybase keypair;
	
	mpz_init(p);
	mpz_init(q);
	make512pq(p,q,keystr,len);
	if(RSAbase_generate(keypair,p,q) != 0){
		*publickey = '\0';
		*secretkey = '\0';
		return -1;
	}
	
	mpz_to_base64str(publickey,86,keypair->n);
	mpz_to_base64str(secretkey,86,keypair->d);
	
	RSAbase_clear(keypair);
	return 0;
}
